' Copyright (c) Microsoft Corporation.  All rights reserved.

'------------------------------------------------------------------------------
' PLEASE NOTE:  Microsoft Corporation (or based on where you live, one of its affiliates) 
' licenses this supplement to you.  You may use it with each validly licensed copy of Microsoft 
' Visual Studio Pre-Release Code-Name "ORCAS" software (the “software”).  You may not use the 
' supplement if you do not have a license for the software.  The license terms for the software 
' apply to your use of this supplement. 
'
' Please note: As this software is distributed in Quebec, Canada, this notice is provided below in French.
' Remarque: Ce logiciel étant distribué au Québec, Canada, les termes de cette licence sont fournis ci-dessous en français.
' REMARQUE:  Microsoft Corporation (ou en fonction du lieu où vous vivez, l’un de ses affiliés) vous 
' accorde une licence pour ce supplément.  Vous pouvez utiliser une copie de ce supplément avec chaque 
' copie concédée sous licence du logiciel Microsoft Visual Studio Pre-Release Code-Name "ORCAS" (le « logiciel »).  
' Vous n’êtes pas autorisé à utiliser ce supplément si vous n’êtes pas titulaire d’une licence pour le logiciel.  
' Les conditions de licence pour le logiciel s’appliquent à l’utilisation de ce supplément.  
'------------------------------------------------------------------------------
Imports System.Data.Objects
Imports NorthwindLib


Public Class WinAppSample

    ' declare a form-wide (or app wide) object context. Note
    ' that there is no need to pass connection information, it's
    ' picked up from configuration file based on the name of the
    ' schema
    Dim _db As New Northwind()

    ' track previously selected category and product objects so
    ' we can implement delete
    Dim _previousCategory As Category
    Dim _currentCategory As Category
    Dim _previousProduct As Product
    Dim _currentProduct As Product

    Private Sub WinAppSample_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        LoadCategories()
    End Sub

    ' the current category changed, the products for this category from
    ' the category object. We'll load only if we didn't load before.
    Private Sub CategoryBindingSource_CurrentChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CategoryBindingSource.CurrentChanged
        ' track the previously selected category so we can do deletes
        _previousCategory = _currentCategory
        _currentCategory = CType(CategoryBindingSource.Current, Category)

        ' load the products if we didn't load them before
        If Not _currentCategory.Products.IsLoaded Then

            If _currentCategory.EntityState = DataRowState.Modified Or _
               _currentCategory.EntityState = DataRowState.Unchanged Then
                ' NOTE: this does a database round-trip and is a common failure point
                _currentCategory.Products.Load()
            End If
        End If

        ' bind the products collection to the products binding source
        ProductBindingSource.DataSource = _currentCategory.Products.Source.Where("it is of (only NorthwindLib.Product)")

    End Sub

    Private Sub CategoryBindingSource_AddingNew(ByVal sender As System.Object, ByVal e As System.ComponentModel.AddingNewEventArgs) Handles CategoryBindingSource.AddingNew
        ' the user just added a new category row to the grid, let the context know, so when
        ' we call SaveChanges the category object gets pushed to the database.
        Dim newcat As New Category()
        e.NewObject = newcat
        _db.AddObject(newcat)
    End Sub



    Private Sub ProductBindingSource_CurrentChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProductBindingSource.CurrentChanged
        ' track current and previous selection so we know which one
        ' to delete when we get an event
        _previousProduct = _currentProduct
        _currentProduct = CType(ProductBindingSource.Current, Product)
    End Sub

    Private Sub ProductBindingSource_AddingNew(ByVal sender As System.Object, ByVal e As System.ComponentModel.AddingNewEventArgs) Handles ProductBindingSource.AddingNew
        ' the user just added a new product. We need to relate the product to the
        ' current category. Note that since we're adding it to the related collection
        ' in the category object, we don't need to also call _db.AddObject()
        Dim cat As Category = CType(CategoryBindingSource.Current, Category)
        Dim newprod As New Product()
        e.NewObject = newprod
        cat.Products.Add(newprod)
    End Sub



    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Application.Exit()
    End Sub

    Private Sub SaveChangesToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChangesToolStripMenuItem.Click
        ' user requested that we save changes that have been made
        _db.SaveChanges()
    End Sub

    Private Sub btnRefreshCategories_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefreshCategories.Click
        LoadCategories()
        CategoryDataGridView.Refresh()
    End Sub

    Private Sub btnRefreshProducts_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefreshProducts.Click
        Dim cat As Category = CType(CategoryBindingSource.Current, Category)

        If cat IsNot Nothing Then
            ' call Load() again to refresh the list of products related to 
            ' the current category
            cat.Products.Load()
            ProductDataGridView.Refresh()
        End If
    End Sub

    Sub LoadCategories()
        ' fill the categories grid
        Dim categories As ObjectQuery(Of Category) = _db.CreateQuery(Of Category)("Select value c from Categories as c order by c.CategoryName")
        ' turn the results into a bindable-list and bind it to the source        
        CategoryBindingSource.DataSource = categories
    End Sub



End Class
